@@ -27,15 +27,44 @@
 #  USA.
 #
 
-segm3D <- function(y,lkern="Gaussian",weighted=TRUE,
+segm3D <- function(y,weighted=TRUE,
                    sigma2=NULL,mask=NULL,hinit=NULL,hmax=NULL,
                    ladjust=1,graph=FALSE,wghts=NULL,
-                   spmin=.3,h0=c(0,0,0),res=NULL, resscale=NULL, ddim=NULL,thresh=3.5,delta=0,fov=NULL) {
+                   df=100,h0=c(0,0,0),res=NULL, resscale=NULL, 
+                   ddim=NULL,delta=0,fov=NULL,alpha=.05) {
 #
-# qlambda, corrfactor adjusted for case lkern="Gaussian",skern="Plateau" only
 #
 #  Auxilary functions
    IQRdiff <- function(y) IQR(diff(y))/1.908
+   getkrval <- function(df,ladj,n,kstar,alpha){
+#
+#    this delivers an upper bound for kritical values over a wide range of parameters
+#    covering the typical situations in fMRI
+#    n in (32^3 : 64^2*32)
+#    df in (40 : 250)
+#    ladj in (1 : 1.4) 
+#    kstar in (10:27)  corrsponding to maximal bandwidths  2.5 - 5
+#    see file sim_fmri_kritval.r in R/segmentation/fmrikrv/
+      dfinv <- log(df)-30000/df^1.44
+      explvar <- c(1, dfinv, log(n), kstar, dfinv*log(n), dfinv*(ladj-1))
+      a <- c(1, alpha, alpha^2, sqrt(alpha))
+      acoef <- matrix(c(1           ,  0           ,  0          ,  0           ,
+                        0.0002129201, -0.000820306 ,  0          ,  0           ,
+                        0.0016314442,  0.0061321878,  0          , -0.0043558989,
+                        0.0001558305,  0.0007918858,  0          , -0.0006520962,
+                       -0.0001535607,  0.0009159794, -0.004950741,  0           ,
+                        0.0003009761, -0.0017770459,  0.013396270,  0   ),4,6)
+      dimnames(acoef) <- list(c("(Intercept)","a","a2","ah"),NULL)
+      ecoefs <- t(acoef)%*%a
+      t(explvar)%*%ecoefs
+   }
+fovcorr <- function(h,fwhm=TRUE){
+# correction factor for FOV in case of spatial correlation
+fwhm2bw <- function(hfwhm) hfwhm/sqrt(8*log(2))
+if(fwhm) h <- fwhm2bw(h)
+ 1-0.3181*log(h[1]+1)-0.3189*log(h[2]+1)+
+                       0.1025*log(h[1]+1)*log(h[2]+1)
+}
 #
 # first check arguments and initialize
 #
@@ -54,35 +83,20 @@ segm3D <- function(y,lkern="Gaussian",weighted=TRUE,
       stop("y has to be 3 dimensional")
    }
 # set the code for the kernel (used in lkern) and set lambda
-   lkern <- switch(lkern,
-                   Triangle=2,
-                   Plateau=1,
-                   Gaussian=3,
-                   1)
+   lkern <- 1
    skern <- 1
-# define qlambda, lambda
-   qlambda <- switch(skern,.9945,.9988,.9995)
-   lambda <- ladjust*qchisq(qlambda,1)
+# define lambda
+   lambda <- ladjust*16
 # to have similar preformance compared to skern="Exp"
-   lambda <- 4/3*lambda
-   if(is.null(spmin)) spmin <- .3 else spmin <- 0
+   spmin <- .25 
    spmax <- 1
    if (is.null(hinit)||hinit<1) hinit <- 1
   
 # define hmax
    if (is.null(hmax)) hmax <- 5    # uses a maximum of about 520 points
 
-# re-define bandwidth for Gaussian lkern!!!!
-   if (lkern==3) {
-# assume  hmax was given in  FWHM  units (Gaussian kernel will be truncated at 4)
-      hmax <- fwhm2bw(hmax)*4
-      hinit <- min(hinit,hmax)
-   }
-   if (qlambda == 1) hinit <- hmax
 
 # define hincr
-# determine corresponding bandwidth for specified correlation
-   if(is.null(h0)) h0 <- rep(0,3)
 
 # estimate variance in the gaussian case if necessary  
 # deal with homoskedastic Gaussian case by extending sigma2
@@ -105,49 +119,48 @@ segm3D <- function(y,lkern="Gaussian",weighted=TRUE,
                            as.integer(ddim[4]),
                            var = double(n1*n2*n3),
                            PACKAGE="fmri",DUP=FALSE)$var
-   lambda <- lambda*2   
+   varest0 <- vartheta0
+   vq <- varest0*sigma2
 # Initialize  list for bi and theta
    if (is.null(wghts)) wghts <- c(1,1,1)
    hinit <- hinit/wghts[1]
    hmax <- hmax/wghts[1]
-   wghts <- (wghts[2:3]/wghts[1])
-   tobj <- list(bi= rep(1,n))
+   wghts <- wghts[2:3]/wghts[1]
+#   tobj <- list(bi= rep(1,n))
+   tobj <- list(bi= sigma2)
    theta <- y
    segm <- array(0,dy[1:3])
-   varest <- 1/sigma2
+   varest <- varest0
    maxvol <- getvofh(hmax,lkern,wghts)
-   if(is.null(fov)) fov <- prod(ddim[1:3])
+   if(is.null(fov)) fov <- sum(mask)
+   fov <- fovcorr(h0)*fov
    kstar <- as.integer(log(maxvol)/log(1.25))
    steps <- kstar+1
-   cat("FOV",fov,"delta",delta,"thresh",thresh,"ladjust",ladjust,"lambda",lambda,"\n")
-   if(qlambda < 1) k <- 1 else k <- kstar
+   k <- 1 
    hakt <- hinit
    hakt0 <- hinit
    lambda0 <- lambda
    if (hinit>1) lambda0 <- 1e50 # that removes the stochstic term for the first step
    scorr <- numeric(3)
-   if(h0[1]>0) scorr[1] <-  get.corr.gauss(h0[1],2)
-   if(h0[2]>0) scorr[2] <-  get.corr.gauss(h0[2],2)
-   if(h0[3]>0) scorr[3] <-  get.corr.gauss(h0[3],2)
    total <- cumsum(1.25^(1:kstar))/sum(1.25^(1:kstar))
+   thresh <- 1
+   for(i in 10:kstar) thresh <- max(thresh,getkrval(df,ladjust,fov,i,alpha))
+#  just to ensure monotonicity of thresh with kmax, there exist a few parameter configurations
+#  where the approximation formula does not ensure monotonicity
+   cat("FOV",fov,"delta",delta,"thresh",thresh,"ladjust",ladjust,"lambda",lambda,"df",df,"\n")
 # run single steps to display intermediate results
+   residuals <- residuals*resscale
+#
+#   need these values to compute variances 
+#
    while (k<=kstar) {
       hakt0 <- gethani(1,10,lkern,1.25^(k-1),wghts,1e-4)
       hakt <- gethani(1,10,lkern,1.25^k,wghts,1e-4)
-      hakt.oscale <- if(lkern==3) bw2fwhm(hakt/4) else hakt
-      cat("step",k,"bandwidth",signif(hakt.oscale,3)," ")
+      cat("step",k,"bandwidth",signif(hakt,3)," ")
       dlw <- (2*trunc(hakt/c(1,wghts))+1)[1:d]
-#  need bandwidth in voxel for Spaialvar.gauss, h0 is in voxel
-      if (any(h0>0)) lambda0 <- lambda0 * Spatialvar.gauss(bw2fwhm(hakt0)/4/c(1,wghts),h0,d)/
-      Spatialvar.gauss(h0,1e-5,d)/Spatialvar.gauss(bw2fwhm(hakt0)/4/c(1,wghts),1e-5,d)
-# Correction C(h0,hakt) for spatial correlation depends on h^{(k-1)}  all bandwidth-arguments in FWHM 
       hakt0 <- hakt
       theta0 <- theta
       bi0 <- tobj$bi
-#
-#   need these values to compute variances after the last iteration
-#
-      residuals <- readBin(res,"integer",prod(ddim),2)*resscale
       tobj <- .Fortran("segm3d",
                        as.double(y),
                        as.double(residuals),
@@ -158,44 +171,38 @@ segm3D <- function(y,lkern="Gaussian",weighted=TRUE,
                        as.integer(n2),
                        as.integer(n3),
                        as.integer(nt),
+                       as.double(df),
                        hakt=as.double(hakt),
                        as.double(lambda0),
                        as.double(theta0),
                        bi=as.double(bi0),
                        thnew=double(n1*n2*n3),
-                       as.integer(lkern),
-                       as.double(spmin),
-                       as.double(spmax),
                        double(prod(dlw)),
                        as.double(wghts),
                        double(nt),#swres
-                       as.double(delta),
-                       pvalue=double(n1*n2*n3),#pvalue
+                       double(n1*n2*n3),#pvalues
                        segm=as.integer(segm),
+                       as.double(delta),
                        as.double(thresh),
+                       as.integer(k),
                        as.double(fov),
+                       as.double(vq),
+                       as.double(varest0),
                        varest=as.double(varest),
-                       PACKAGE="fmri",DUP=FALSE)[c("bi","thnew","hakt","segm","varest","pvalue")]
-
+                       PACKAGE="fmri",DUP=FALSE)[c("bi","thnew","hakt","segm","varest")]
       gc()
       theta <- array(tobj$thnew,dy[1:3]) 
       segm <- array(tobj$segm,dy[1:3])
-#      par(mfrow=c(1,3))
-#      plot(density(tobj$pvalue))
-#      if(sum(tobj$segm==1)>10) plot(density(tobj$pvalue[tobj$segm==1]))
-#      image(apply(segm>0,1:2,sum),zlim=c(0,26))
-#      cat("\n",sum(signal&(segm==1)),sum(signal&(segm!=1)),sum(!signal&(segm==1)),sum(!signal&(segm!=1)),"\n")
-#      readline("Press enter")
-      varest <- array(tobj$varest/vartheta0/sigma2,dy[1:3])
+      varest <- array(tobj$varest,dy[1:3])
       dim(tobj$bi) <- dy[1:3]
       if (graph) {
          par(mfrow=c(2,2),mar=c(1,1,3,.25),mgp=c(2,1,0))
          image(y[,,n3%/%2+1],col=gray((0:255)/255),xaxt="n",yaxt="n")
          title(paste("Observed Image  min=",signif(min(y),3)," max=",signif(max(y),3)))
          image(theta[,,n3%/%2+1],col=gray((0:255)/255),xaxt="n",yaxt="n")
-         title(paste("Reconstruction  h=",signif(hakt.oscale,3)," min=",signif(min(theta),3),"   max=",signif(max(theta),3)))
+         title(paste("Reconstruction  h=",signif(hakt,3)," min=",signif(min(theta),3),"   max=",signif(max(theta),3)))
          image(segm[,,n3%/%2+1]>0,col=gray((0:255)/255),xaxt="n",yaxt="n")
-         title(paste("Segmentation  h=",signif(hakt.oscale,3)," detected=",sum(segm>0)))
+         title(paste("Segmentation  h=",signif(hakt,3)," detected=",sum(segm>0)))
          image(tobj$bi[,,n3%/%2+1],col=gray((0:255)/255),xaxt="n",yaxt="n")
          title(paste("Sum of weights: min=",signif(min(tobj$bi),3)," mean=",signif(mean(tobj$bi),3)," max=",signif(max(tobj$bi),3)))
       }
@@ -204,17 +211,14 @@ segm3D <- function(y,lkern="Gaussian",weighted=TRUE,
       }
       k <- k+1
 #  adjust lambda for the high intrinsic correlation between  neighboring estimates 
-      c1 <- (prod(h0+1))^(1/3)
-      c1 <- 2.7214286 - 3.9476190*c1 + 1.6928571*c1*c1 - 0.1666667*c1*c1*c1
-      x <- (prod(1.25^(k-1)/c(1,wghts)))^(1/3)
-      scorrfactor <- (c1+x)/(c1*prod(h0+1)+x)
-      lambda0 <- lambda*scorrfactor
+      lambda0 <- lambda
       gc()
    }
 
   z <- list(theta=theta,ni=tobj$bi,var=varest,y=y,segm=segm,
             hmax=tobj$hakt*switch(lkern,1,1,bw2fwhm(1/4)),
-            call=args,scorr=scorr)
+            call=args,scorr=scorr,mask=mask)
   class(z) <- "aws.gaussian"
   invisible(z)
 }
+
